package cn.com.jdkj.aspect;

import cn.com.jdkj.util.SpringUtils;
import cn.hutool.core.stream.StreamUtil;
import cn.hutool.json.JSONUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindingResult;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.InetAddress;
import java.net.URLDecoder;
import java.net.UnknownHostException;
import java.text.MessageFormat;
import java.time.Duration;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 描述
 * 日志切面操作
 **/
@Aspect
@Component
public class LogAspect {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    private boolean noLog = false;


    @Pointcut("execution(* cn.com.jdkj.customer.controller.*.*(..))")//两个..代表所有子目录，最后括号里的两个..代表所有参数
    public void logPointCut() {
    }

    @Before("logPointCut()")
    public void doBefore(JoinPoint joinPoint) {

        String methodName = joinPoint.getSignature().getName();
        String filterStr = "initDataBinder";
        if (methodName.contains(filterStr)) {
            noLog = true;
            return;
        }
        // 接收到请求，记录请求内容
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        // 记录下请求内容
        logger.info("******************** 接口调用请求 - 开始 ********************");
        logger.info("************ 当前运行环境 {} ***************", SpringUtils.getActiveProfile());
        logger.info("######Header信息:{}", getHeaders(request));
        logger.info("###### 接口信息 : {} {}", request.getMethod(), request.getRequestURL().toString());
        Object[] args = joinPoint.getArgs();
        List<Object> logArgs = StreamUtil.of(args).filter(arg -> (!(arg instanceof HttpServletRequest) && !(arg instanceof HttpServletResponse)
                && !(arg instanceof MultipartFile) && !(arg instanceof BindingResult))).collect(Collectors.toList());
        logger.info("###### 参数 : " + JSONUtil.toJsonStr(logArgs));
        logger.info("###### IP : " + request.getRemoteAddr());
        logger.info("###### CLASS_METHOD : " + joinPoint.getSignature().getDeclaringTypeName() + "."
                + joinPoint.getSignature().getName());
        noLog = false;

    }

    public String getHeaders(HttpServletRequest request) {
        StringBuilder builder = new StringBuilder();
        Enumeration<String> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String name = headerNames.nextElement();
            String header = request.getHeader(name);
            if (builder.length() > 0) {
                builder.append(";");
            }
            builder.append(name).append(": ").append(header);
        }
        return builder.toString();
    }

    @AfterReturning(returning = "ret", pointcut = "!@annotation(cn.com.jdkj.annotation.LogIgnore) && logPointCut()")
// returning的值和doAfterReturning的参数名一致
    public void doAfterReturning(Object ret) {
        if (noLog) {
            noLog = false;
            return;
        }
        // 处理完请求，返回内容
        logger.info("###### 返回值 : " + ret);
        logger.info("******************** 接口调用请求 - 结束 ********************");
    }

    @Around("logPointCut()")
    public Object doAround(ProceedingJoinPoint pjp) throws Throwable {

        Instant start = Instant.now();
        Object ob = pjp.proceed();
        if (!noLog) {
            logger.info(MessageFormat.format("###### 接口调用耗时 : {0}ms", Duration.between(start, Instant.now()).toMillis()));
        }
        return ob;
    }

    public String getIpAddress(HttpServletRequest request) {
        String ipAddress = request.getHeader("x-forwarded-for");
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getRemoteAddr();
            String localIp = "127.0.0.1";
            String localIpv6 = "0:0:0:0:0:0:0:1";
            if (ipAddress.equals(localIp) || ipAddress.equals(localIpv6)) {
                // 根据网卡取本机配置的IP
                InetAddress inet;
                try {
                    inet = InetAddress.getLocalHost();
                    ipAddress = inet.getHostAddress();
                } catch (UnknownHostException e) {
                    e.printStackTrace();
                }
            }
        }
        // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        String ipSeparate = ",";
        int ipLength = 15;
        if (ipAddress != null && ipAddress.length() > ipLength) {
            if (ipAddress.indexOf(ipSeparate) > 0) {
                ipAddress = ipAddress.substring(0, ipAddress.indexOf(ipSeparate));
            }
        }
        return ipAddress;
    }




}
